package com.example.qualitychecker.store;

import com.example.qualitychecker.apiservice.ApiService;
import com.example.qualitychecker.apiservice.ApiServiceFactory;
import com.example.qualitychecker.models.Dimension;
import com.example.qualitychecker.models.EmployeePhoto;
import com.example.qualitychecker.models.InspectionCheckPoint;
import com.example.qualitychecker.models.InspectionDefect;
import com.example.qualitychecker.models.InspectionDimension;
import com.example.qualitychecker.models.InspectionItemCheckPoint;
import com.example.qualitychecker.models.InspectionItemDimension;
import com.example.qualitychecker.models.InspectionReport;
import com.example.qualitychecker.models.InspectionReportItem;
import com.example.qualitychecker.models.ItemUnit;
import com.example.qualitychecker.models.Product;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class Store {

    private static Store instance;
    private InspectionReport report;
    private ApiService apiService;
    private List<InspectionCheckPoint> checkPoints;
    private List<InspectionDefect> defects;
    private List<InspectionDimension> dimensionList;
    private List<InspectionItemDimension> dimensions;
    private List<Product> products;
    private EmployeePhoto employeePhoto;
    private List<ItemUnit> itemUnits;

    private Store(){
    }

    public static synchronized Store getInstance(){
        if( instance == null ){
            instance = new Store();
        }
        return instance;
    }

    public synchronized void initializeNewReport() {
        checkPoints = new ArrayList<>();
        defects = new ArrayList<>();
        dimensions = new ArrayList<>();
        products = new ArrayList<>();
        itemUnits = new ArrayList<>();
        dimensionList = new ArrayList<>();
        apiService = ApiServiceFactory.getApiService();
        report = new InspectionReport();
        InspectionReportItem item = new InspectionReportItem();
        item.setDimensions( new ArrayList<>() );
        report.setItems(Collections.singletonList( item ));

        apiService.fetchCheckPoints().enqueue(
                new Callback<List<InspectionCheckPoint>>() {
                    @Override
                    public void onResponse( Call<List<InspectionCheckPoint>> call,
                                            Response<List<InspectionCheckPoint>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            checkPoints =  response.body() ;
                        }
                    }
                    @Override
                    public void onFailure(Call<List<InspectionCheckPoint>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );

        apiService.fetchDefects().enqueue(
                new Callback<List<InspectionDefect>>() {
                    @Override
                    public void onResponse( Call<List<InspectionDefect>> call,
                                            Response<List<InspectionDefect>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            defects = response.body();
                        }
                    }
                    @Override
                    public void onFailure( Call<List<InspectionDefect>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );


        apiService.fetchCheckPoints().enqueue(
                new Callback<List<InspectionCheckPoint>>() {
                    @Override
                    public void onResponse(Call<List<InspectionCheckPoint>> call,
                                           Response<List<InspectionCheckPoint>> response) {

                        if( response.isSuccessful() && response.body() != null ){
                            List<InspectionItemCheckPoint> itemCp = new ArrayList<>();
                            for( InspectionCheckPoint cp : response.body() ){
                                InspectionItemCheckPoint icp = new InspectionItemCheckPoint();
                                icp.setTitle( cp.getTitle() );
                                icp.setDefectSeverity("MINOR");
                                icp.setChecked( false );
                                icp.setRemarks("");
                                itemCp.add( icp );
                            }
                            report.getItems().get( 0 ).setCheckPoints( itemCp );
                        }
                    }
                    @Override
                    public void onFailure(Call<List<InspectionCheckPoint>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );

        apiService.fetchItemUnits().enqueue(
                new Callback<List<ItemUnit>>() {
                    @Override
                    public void onResponse(Call<List<ItemUnit>> call,
                                           Response<List<ItemUnit>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            System.out.println( response.body() );
                            itemUnits = response.body();
                        }
                    }

                    @Override
                    public void onFailure(Call<List<ItemUnit>> call,
                                          Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );

        apiService.fetchDimensions().enqueue(
                new Callback<List<InspectionDimension>>() {
                    @Override
                    public void onResponse(Call<List<InspectionDimension>> call, Response<List<InspectionDimension>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            dimensionList = response.body();
                        }
                    }

                    @Override
                    public void onFailure(Call<List<InspectionDimension>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );
    }

    public InspectionReport getReport() {
        return report;
    }

    public void setReport(InspectionReport report) {
        this.report = report;
    }

    public List<InspectionCheckPoint> getCheckPoints() {
        return checkPoints;
    }

    public List<InspectionDefect> getDefects() {
        return defects;
    }

    public List<InspectionItemDimension> getDimensions() {
        return dimensions;
    }

    public void setDimensions(List<InspectionItemDimension> dimensions) {
        this.dimensions = dimensions;
    }

    public List<Product> getProducts() {
        return products;
    }

    public void setProducts(List<Product> products) {
        this.products = products;
    }

    public EmployeePhoto getEmployeePhoto() {
        return employeePhoto;
    }

    public void setEmployeePhoto(EmployeePhoto employeePhoto) {
        this.employeePhoto = employeePhoto;
    }

    public List<String> getItemUnits() {
        return itemUnits.stream().map( ItemUnit::getTitle ).collect(Collectors.toList());
    }

    public void setItemUnits(List<ItemUnit> itemUnits) {
        this.itemUnits = itemUnits;
    }

    public String getProfileImage(  ) {
        if( employeePhoto != null  ){
            return "https://portal.utopiaindustries.pk/uind/employee-photo/" + employeePhoto.getEmployeeId() +".jpeg";
        }
        return "";
    }

    public List<String> getDimensionList() {
        return dimensionList.stream().map( InspectionDimension::getTitle ).collect(Collectors.toList());
    }

    public void setDimensionList(List<InspectionDimension> dimensionList) {
        this.dimensionList = dimensionList;
    }

    @Override
    public String toString() {
        return "Store{" +
                "report=" + report +
                ", apiService=" + apiService +
                ", checkPoints=" + checkPoints +
                ", defects=" + defects +
                '}';
    }
}
