package com.example.qualitychecker.ui.activities;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.example.qualitychecker.R;
import com.example.qualitychecker.apiservice.ApiService;
import com.example.qualitychecker.apiservice.ApiServiceFactory;
import com.example.qualitychecker.models.EmployeePhoto;
import com.example.qualitychecker.models.InspectionReport;
import com.example.qualitychecker.store.Store;
import com.example.qualitychecker.ui.adapter.ReportAdapter;

import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class HomeActivity extends AppCompatActivity implements View.OnClickListener {

    private Button createReportBtn,refreshReportsBtn,logoutBtn;
    private RecyclerView recyclerView;
    private ApiService apiService;
    private TextView usernameTitle;
    private ImageView profileImage;
    private Store store;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        if( ! isNetworkConnected() ){
            Toast.makeText( this, "No Internet Connection", Toast.LENGTH_LONG ).show();
        }
        setContentView(R.layout.activity_home);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });
        setRequestedOrientation( ActivityInfo.SCREEN_ORIENTATION_PORTRAIT );

        // initialize views
        store = Store.getInstance();
        apiService = ApiServiceFactory.getApiService();
        createReportBtn = findViewById( R.id.create_report_btn );
        refreshReportsBtn = findViewById( R.id.refresh_btn );
        logoutBtn = findViewById( R.id.logout_btn );
        recyclerView = findViewById( R.id.reports_recyclerview );
        usernameTitle = findViewById( R.id.username_title );
        profileImage = findViewById( R.id.profile_image );

        refreshReportsBtn.setOnClickListener( this );
        createReportBtn.setOnClickListener( this );
        logoutBtn.setOnClickListener( this );

        recyclerView.setLayoutManager( new LinearLayoutManager( this ) );

        updateProfileImage();
        refreshReportData();
    }

    private void refreshReportData(){
        apiService.fetchAllReports().enqueue(
                new Callback<List<InspectionReport>>() {
                    @Override
                    public void onResponse(Call<List<InspectionReport>> call, Response<List<InspectionReport>> response) {
                        if( response.isSuccessful() && response.body() != null ){
                            System.out.println( response.body() );
                            ReportAdapter adapter = new ReportAdapter( response.body() );
                            recyclerView.setAdapter( adapter );
                        } else {
                            System.out.println("Error");
                        }
                    }
                    @Override
                    public void onFailure(Call<List<InspectionReport>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if( id == R.id.create_report_btn ){
            Intent intent = new Intent( this, MainActivity.class );
            startActivity( intent );
        }
        if( id == R.id.logout_btn ){
            SharedPreferences sharedPreferences = getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putString("username", null );
            editor.putString("password", null );
            editor.apply();
            Intent intent = new Intent( HomeActivity.this, LoginActivity.class );
            startActivity( intent );
            finish();
        }
        if( id == R.id.refresh_btn ){
            refreshReportData();
            updateProfileImage();
        }
    }

    private void updateProfileImage(){
        SharedPreferences sharedPreferences = getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
        String username = sharedPreferences.getString("username", null );
        System.out.println( );
        if( username != null ){
            apiService.fetchEmployeePhoto( username ).enqueue(
                    new Callback<EmployeePhoto>() {
                        @Override
                        public void onResponse(Call<EmployeePhoto> call, Response<EmployeePhoto> response) {
                            System.out.println( response.body() );
                            if( response.isSuccessful() && response.body() != null ){
                                System.out.println( response.body() );
                                usernameTitle.setText( response.body().getName() );
                                store.setEmployeePhoto( response.body() );
                                Glide.with( getBaseContext() )
                                        .load( store.getProfileImage( ) )
                                        .into( profileImage );
                            }
                        }

                        @Override
                        public void onFailure(Call<EmployeePhoto> call, Throwable t) {
                            System.out.println( t.getMessage() );
                        }
                    }
            );

        }
    }
    private boolean isNetworkConnected() {
        ConnectivityManager connectivityManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = connectivityManager.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }

    @Override
    protected void onResume() {
        super.onResume();
        refreshReportData();
    }
}