package com.example.qualitychecker.ui.adapter;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.qualitychecker.R;
import com.example.qualitychecker.models.InspectionCheckPoint;
import com.example.qualitychecker.models.InspectionDefect;
import com.example.qualitychecker.models.InspectionItemCheckPoint;

import java.util.List;
import java.util.stream.Collectors;

public class CheckPointAdapter extends
        RecyclerView.Adapter<CheckPointAdapter.ViewHolder> {

    private final List<InspectionItemCheckPoint> checkPointList;
    private final List<InspectionCheckPoint> checkPoints;
    private final List<InspectionDefect> defects;
    private final List<String> defectTypes;
    private final Context context;
    public CheckPointAdapter(  List<InspectionItemCheckPoint> checkPointList,
                               List<InspectionCheckPoint> checkPoints,
                               List<InspectionDefect> defects,
                               List<String> defectTypes,
                               Context context ) {

        this.checkPointList = checkPointList;
        this.checkPoints = checkPoints;
        this.defects = defects;
        this.defectTypes = defectTypes;
        this.context = context;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from( parent.getContext() )
                .inflate(R.layout.check_point, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        InspectionItemCheckPoint checkPoint = checkPointList.get( position );
        holder.bind(checkPoints, defects, defectTypes, checkPoint);
        // spinner on change
        {
            holder.checkpointSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String selectedItem = parent.getItemAtPosition(position).toString();
                    InspectionCheckPoint cp = checkPoints.stream().filter(check -> check.getTitle().equalsIgnoreCase(selectedItem))
                            .findFirst().orElse(new InspectionCheckPoint());
                    holder.populateDefects(cp, defects, checkPoint);
                    checkPoint.setTitle(selectedItem);
                }
                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });

            holder.defectsSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String selectedItem = parent.getItemAtPosition(position).toString();
                    checkPoint.setDefectTitle(selectedItem);
                }
                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });

            holder.defectTypeSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String selectedItem = parent.getItemAtPosition(position).toString();
                    checkPoint.setDefectSeverity(selectedItem);
                }
                @Override
                public void onNothingSelected(AdapterView<?> parent) {

                }
            });
        }
        // checkboxes on change
        {
            holder.okCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
                if (isChecked) {
                    holder.linearLayout.setVisibility(View.INVISIBLE);
                }
                checkPoint.setChecked ( false );
                holder.noCheckBox.setChecked( !isChecked );
            });

            holder.noCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
                if (isChecked) {
                    holder.linearLayout.setVisibility(View.VISIBLE);
                }
                checkPoint.setChecked( true );
                holder.okCheckBox.setChecked( !isChecked );
            });
        }
        // remarks on change
        {
            holder.remarks.addTextChangedListener(new TextWatcher() {
                @Override
                public void beforeTextChanged(CharSequence s, int start, int count, int after) {

                }

                @Override
                public void onTextChanged(CharSequence s, int start, int before, int count) {
                    checkPoint.setRemarks( s.toString() );
                }

                @Override
                public void afterTextChanged(Editable s) {

                }
            });
        }

        holder.imagePicker.setOnClickListener( v -> {
            Intent intent = new Intent( Intent.ACTION_GET_CONTENT );
            intent.addCategory( Intent.CATEGORY_OPENABLE );
            intent.setType( "image/*" );
            ( ( Activity ) context ).startActivityForResult( intent,  position );
        });
    }


    @Override
    public int getItemCount() {
        return checkPointList.size();
    }

    public static class ViewHolder extends RecyclerView.ViewHolder {
        private final Spinner checkpointSpinner, defectsSpinner, defectTypeSpinner;
        private final CheckBox okCheckBox, noCheckBox;
        private final EditText remarks;
        private final ImageButton imagePicker;
        private final LinearLayout linearLayout;
        private final TextView selectedImage;

        public ViewHolder(  @NonNull View itemView) {
            super(itemView);
            checkpointSpinner = itemView.findViewById(R.id.check_point_spinner);
            okCheckBox = itemView.findViewById(R.id.check_point_ok);
            noCheckBox = itemView.findViewById(R.id.check_point_no);
            remarks = itemView.findViewById(R.id.check_point_remarks);
            imagePicker = itemView.findViewById(R.id.image_picker );
            selectedImage = itemView.findViewById( R.id.selected_image );
            defectsSpinner = itemView.findViewById(R.id.defect_spinner);
            defectTypeSpinner = itemView.findViewById(R.id.defect_type_spinner);
            linearLayout = itemView.findViewById(R.id.defect_layout);
        }
        public void bind(List<InspectionCheckPoint> checkPoints,
                         List<InspectionDefect> defects,
                         List<String> defectTypeOptions,
                         InspectionItemCheckPoint data ) {
            List<String> checkPointsList = checkPoints.stream().map(InspectionCheckPoint::getTitle).collect(Collectors.toList());

            // Populate checklist Spinner dropdown
            ArrayAdapter<String> spinnerAdapter = new ArrayAdapter<>(checkpointSpinner.getContext(),
                    android.R.layout.simple_spinner_item, checkPointsList);
            spinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            checkpointSpinner.setAdapter(spinnerAdapter);

            // Populate defect type Spinner dropdown
            ArrayAdapter<String> defectTypeSpinnerAdapter = new ArrayAdapter<>(defectTypeSpinner.getContext(),
                    android.R.layout.simple_spinner_item, defectTypeOptions);
            defectTypeSpinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            defectTypeSpinner.setAdapter(defectTypeSpinnerAdapter);

            // Pre populate
            String defaultSelectedItem = data.getTitle();
            int defaultPosition = checkPointsList.indexOf(defaultSelectedItem);
            checkpointSpinner.setSelection(defaultPosition);

            String defaultSelectedDefectType = data.getDefectSeverity();
            int defaultSelectedDefectTypePosition = defectTypeOptions.indexOf(defaultSelectedDefectType);
            defectTypeSpinner.setSelection(defaultSelectedDefectTypePosition);

            remarks.setText( data.getRemarks() );
            okCheckBox.setChecked( data.getChecked() );
            noCheckBox.setChecked( ! data.getChecked());

            if( data.getChecked()){
                linearLayout.setVisibility(View.INVISIBLE);
            } else {
                linearLayout.setVisibility(View.VISIBLE);
            }

            selectedImage.setText( data.getImagePath() );
        }
        private void populateDefects(InspectionCheckPoint cp, List<InspectionDefect> defects, InspectionItemCheckPoint data) {

            List<InspectionDefect> filteredDefects = defects.stream()
                    .filter(d -> d.getCategory().equalsIgnoreCase(cp.getCategory()))
                    .collect(Collectors.toList());
            List<String> defectList = filteredDefects.stream().map(InspectionDefect::getDefect).collect(Collectors.toList());
            // Populate defect Spinner dropdown
            ArrayAdapter<String> defectSpinnerAdapter = new ArrayAdapter<>(defectsSpinner.getContext(),
                    android.R.layout.simple_spinner_item, defectList);
            defectSpinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            defectsSpinner.setAdapter(defectSpinnerAdapter);

            String defaultDefectSelectedItem = data.getDefectTitle();
            int defaultSelectedDefectPosition = defectList.indexOf(defaultDefectSelectedItem);
            defectsSpinner.setSelection(defaultSelectedDefectPosition);
        }
    }
}
